function [yyyyd,xxyyd,xxxxd, ZZ, gx,hx, MM, gxz] = varprio(x,f,fx, fy, fxp, fyp,statevar_cu,controlvar_cu);
%% description:
%% solution to DSGE model - delivers transition equation for the state variables  S_t
%% transition equation: S_t = TC + TTT S_{t-1} + RRR eps_t, where var(eps_t) = QQ
%% define the measurement equation: X_t = ZZ S_t + D + u_t
%% where var(u_t) = HH, cov(eps_t,u_t) = VV
%% and compute  cross moment matrices for the DSGE prior

%% I present
%% Y_t+1 = ZZ X_t + MM eps_{t+1}  (I assume no measurement errors)
%% where X_t is the state vector with evolution X_{t+1}=hx X_t + eps_{t+1}
%% and then compute  cross moment matrices for the DSGE prior

global nvar nlags ncoint nconst


[ gx, hx, exitflag ] = deep_run(x, f, fx, fy, fxp, fyp);

steady_state();
if exitflag ~= 1 
   yyyyd = 0; xxyyd = 0; xxxxd = 0; 
   return
end

nstate       = length(statevar_cu);
ncontrol     = length(controlvar_cu);

STD_EPSG     = x(end-7);
STD_EPSMUZ   = x(end-6);
STD_EPSMUUPS = x(end-5);
STD_EPSD     = x(end-4);
STD_EPSP     = x(end-3);
STD_EPSW     = x(end-2);
STD_EPSR     = x(end-1);
STD_EPStau   = x(end);

smuz   = find( statevar_cu == 'muz_ba1'    );
smuups = find( statevar_cu == 'muups_ba1'  );
sd     = find( statevar_cu == 'd_ba1'      );
sg     = find( statevar_cu == 'epsg_ba1'   );
sr     = find( statevar_cu == 'mur_ba1'    );
stau   = find( statevar_cu == 'epstau_ba1' );
sp     = find( statevar_cu == 'upsp_ba1'   );
sw     = find( statevar_cu == 'upsw_ba1'   );
%ng     = find([controlvar_cu] == 'epsg_ba1' );
ng     = find([controlvar_cu] == 'g_cu' );
% ng_ba1 = find([statevar_cu] == 'g_ba1' );
% nc_ba1 = find([statevar_cu] == 'c_ba1' );
% ni_ba1 = find([statevar_cu] == 'i_ba1' );
% nw_ba1 = find([statevar_cu] == 'w_ba1' );
nr     = find(controlvar_cu == 'r_cu'     );
ni     = find(controlvar_cu == 'iv_cu'    );
nc     = find(controlvar_cu == 'c_cu'     );
npi    = find(controlvar_cu == 'pai_cu'   );
nw     = find(controlvar_cu == 'w_cu'     );
nh     = find(controlvar_cu == 'h_cu'     );
noutput= find(controlvar_cu == 'output_cu');
ntau   = find(controlvar_cu == 'tau_cu'   );

QQ                = zeros(nstate,nstate);

QQ(sg,sg)         = STD_EPSG^2;  
QQ(smuz,smuz)     = STD_EPSMUZ^2;
QQ(smuups,smuups) = STD_EPSMUUPS^2;
QQ(sd,sd)         = STD_EPSD^2;
QQ(sp,sp)         = STD_EPSP^2;
QQ(sw,sw)         = STD_EPSW^2;
QQ(sr,sr)         = STD_EPSR^2;
QQ(stau,stau)     = STD_EPStau^2;

% HH = gx*QQ*gx';
% VV = QQ*gx';

%series = [ log(GOV(2:end)./GDP(2:end))' log(TR(2:end)./GDP(2:end))' diff(log( [  I' C'  W'])) log(H(2:end))' diff(log(P_GDP'))  FEDFUNDS(2:end) ];
% ZZ(1,:) = gx(ng,:)-gx(noutput,:); %*(hx - eye(size(hx)))+ hx(smuz,:) + 0.3/(1-0.3)*hx(smuups,:);
% ZZ(2,:) = gx(ntau,:);
% ZZ(3,:) = gx(ni,:)*(hx - eye(size(hx)))+ hx(smuz,:) +   1  /(1-THETA)*hx(smuups,:);
% ZZ(4,:) = gx(nc,:)*(hx - eye(size(hx)))+ hx(smuz,:) + THETA/(1-THETA)*hx(smuups,:);
% ZZ(5,:) = gx(nw,:)*(hx - eye(size(hx)))+ hx(smuz,:) + THETA/(1-THETA)*hx(smuups,:);
% ZZ(6,:) = gx(nh,:);
% ZZ(7,:) = 4*gx(npi,:);
% ZZ(8,:) = 4*gx(nr,:);
inn = zeros(nvar, nstate);
% inn([ 1, 3, 4, 6 ],smuz) = 1;
% inn([ 1, 3, 4, 6 ],smuups) = [THETA/(1-THETA) THETA/(1-THETA) 1/(1-THETA) THETA/(1-THETA)];

inn(1,sg) = 1;
inn([ 3, 4, 6 ],smuz)   = 1;
inn([ 3, 4, 6 ],smuups) = [ THETA/(1-THETA) 1/(1-THETA) THETA/(1-THETA)];
%inn([1, 3, 4, 6 ],smuz)   = 1;
%inn([1, 3, 4, 6 ],smuups) = [ THETA/(1-THETA) THETA/(1-THETA) 1/(1-THETA) THETA/(1-THETA)];

% inn(1,:) = hx(smuz,:) + THETA/(1-THETA)*hx(smuups,:);
% inn(3,:) = hx(smuz,:) + THETA/(1-THETA)*hx(smuups,:);
% inn(4,:) = hx(smuz,:) +     1/(1-THETA)*hx(smuups,:);
% inn(6,:) = hx(smuz,:) + THETA/(1-THETA)*hx(smuups,:);
ey = eye(nstate);
ZZ(1,:) = ey(sg,:)*hx;%
%ZZ(1,:) = gx(ng,:)*(hx - ey) + hx(smuz,:)+ THETA/(1-THETA)*hx(smuups,:); %*(hx - eye(size(hx)))+ hx(smuz,:) + 0.3/(1-0.3)*hx(smuups,:);
ZZ(2,:) = gx(ntau,:)*hx;
ZZ(3,:) = gx(nc,:)*(hx - ey)+ hx(smuz,:)+ THETA/(1-THETA)*hx(smuups,:);
ZZ(4,:) = gx(ni,:)*(hx - ey)+ hx(smuz,:)+   1  /(1-THETA)*hx(smuups,:);
ZZ(5,:) = gx(nh,:)*hx;
ZZ(6,:) = gx(nw,:)*(hx - ey)+ hx(smuz,:) + THETA/(1-THETA)*hx(smuups,:);
ZZ(7,:) = 4*gx(npi,:)*hx;
ZZ(8,:) = 4*gx(nr,:)*hx;

%gxz = gx( [ng ntau nc ni nh nw npi nr], : );
%gxz(1,:) = ey(sg,:);
gxz(2:nvar,:) = gx( [ ntau nc ni nh nw npi nr], : );
gxz( 7:8, : ) = 4*gxz( 7:8, : );

EZZ = [ log(MUZSTAR) log(TAU) log(MUZSTAR) log(MUZSTAR*MUUPS) log(MUZSTAR) log(H) 4*log(PAI) 4*log(R) ]';
%EZZ = [log(OUTPUT); log(TAU); log(IV); log(C); log(W) ;log(H); 4*log(PAI) ;4*log(R) ];
if nconst == 1
    EZZZZ = EZZ*EZZ';
elseif nconst == 0
    EZZZZ = zeros(nvar);
end
MM = gxz + inn;

sigx0 = mom_state(hx,QQ,0);

yyyyd = zeros(nvar,nvar);
yyyyd = ZZ*sigx0*ZZ' + EZZZZ + MM*QQ*MM';
xxxxd = zeros(nconst + ncoint + nvar*nlags);
xxyyd = zeros(nconst + ncoint + nvar*nlags,nvar);

if nconst == 1
    xxxxd(ncoint+1,ncoint+1) = 1;
    xxyyd(ncoint+1,:) = EZZ;
end

hxi = hx;
hxi1=eye(size(hx));
for i = 1:nlags
    hlp = EZZZZ + ZZ*hxi*sigx0*ZZ' + ZZ*hxi1*QQ*MM';
    xxyyd(ncoint + nconst + 1 + nvar*(i-1) :ncoint + nconst + nvar*i,:) = transpose(hlp);
    hxi1 = hxi;
    hxi = hxi*hx;
    if nconst == 1
        xxxxd((i-1)*nvar+2+ncoint:i*nvar+1+ncoint,ncoint+1) = EZZ;
        xxxxd(ncoint+1,(i-1)*nvar+2+ncoint:i*nvar+1+ncoint) = EZZ;
    end
    xxxxd(nconst+1+ncoint + (i-1)*nvar :   i*nvar + nconst+ncoint,nconst+1+ncoint + (i-1)*nvar :   i*nvar + nconst+ncoint) = yyyyd;
end
for i = 1:nlags-1
    hxi = hx;
    hxi1 = eye(size(hx));
    for j = i+1:nlags
        hlp = ZZ*hxi*sigx0*ZZ' + ZZ*hxi1*QQ*MM'; 
        xxxxd(nconst+1+ncoint + (i-1)*nvar :   i*nvar + nconst+ncoint, nconst+1+ncoint + (j-1)*nvar : nconst+ncoint + j*nvar) = hlp;
        xxxxd(nconst+1+ncoint + (j-1)*nvar :   nconst+ncoint + j*nvar, nconst+1+ncoint + (i-1)*nvar : nconst+ncoint + i*nvar) = hlp';
        hxi1 = hxi;
        hxi =  hxi*hx;
    end
end
% hxi = eye(nstate);
% for i = nlags-1:-1:1
%     hlp = ZZ*hxi*sigx0*gxz';
%     xxxxd(nconst+ncoint+1+(i-1)*nvar:nconst+ncoint+i*nvar, nconst+ncoint + nlags*nvar+1: nconst+ncoint + nlags*nvar + nvar) = hlp;
%     if (i ~= nlags)
%         xxxxd(nconst+ncoint+1+nlags*nvar:nconst+ncoint+nlags*nvar + nvar, nconst+ncoint+1+(i-1)*nvar:nconst+ncoint+i*nvar)  = hlp';
%     end
%     hxi = hxi*hx;
% end
% xxyyd(nconst+ncoint + nlags*nvar+1:nconst+ncoint + nlags*nvar + nvar,:) = transpose(ZZ*hxi*sigx0*gxz');
% xxxxd(nconst+ncoint + nlags*nvar+1:nconst+ncoint + nlags*nvar + nvar,nconst+ncoint + nlags*nvar+1:nconst+ncoint + nlags*nvar + nvar) = gxz*sigx0*gxz';

% xxxxd(nconst+1+ncoint :   nvar + nconst+ncoint, nconst+1+ncoint:end) = [ZZ*sigx0*ZZ'  ZZ*sigx11*ZZ'  ZZ*sigx12*ZZ'  ZZ*sigx13*ZZ' ] + kron(ones(1,nlags),EZZ*EZZ');
% xxxxd(  nvar+nconst+1+ncoint : 2*nvar + nconst+ncoint, nconst+1+ncoint:end) = [ZZ*sigx9*ZZ' ZZ*sigx0*ZZ'  ZZ*sigx11*ZZ'  ZZ*sigx12*ZZ' ] + kron(ones(1,nlags),EZZ*EZZ');
% xxxxd(2*nvar+nconst+1+ncoint : 3*nvar + nconst+ncoint, nconst+1+ncoint:end) = [ZZ*sigx8*ZZ' ZZ*sigx9*ZZ' ZZ*sigx0*ZZ'  ZZ*sigx11*ZZ' ] + kron(ones(1,nlags),EZZ*EZZ');
% xxxxd(3*nvar+nconst+1+ncoint : 4*nvar + nconst+ncoint, nconst+1+ncoint:end) = [ZZ*sigx7*ZZ' ZZ*sigx8*ZZ' ZZ*sigx9*ZZ' ZZ*sigx0*ZZ' ] + kron(ones(1,nlags),EZZ*EZZ');
%save varprio.mat