% marginal.m
% 
% Marginal Likelihood estimator
% It computes the marginal likelihood of a model given some data
% following the Harmonic Mean procedure proposed by Gelfand and
% Dey (1994). The weighting function follows Geweke (1998). 
%
% It needs the program:
%    fgeweke.m

% Notes:
%    1)The normalization constant in section 4 of the program is
%    model specific and need to recomputed by hand in each case.
%    2)fgeweke (the weighting function) needs to be positive only
%    in the support of the prior (up to computationally relevance).
%
% References:
%    Gelfand, A.E. and D.K. Dey (1994), "Bayesian Model Choice:
%    Asymptotics and Exact Calculations"
%    Journal of the Royal Statistical Society B, 56, pp. 501-514.
%    Geweke, J. (1998), "Using Simulation Methods for Bayesian
%    Econometric Models: Inference, Development, and Communication" 
%    Staff Report 249, Federal Reserve Bank of Minneapolis.
%    
% Pau Rabanal and Juan Rubio-Ramirez
% New York and Minneapolis, 6-21-2001

clear all
close all
clc

[ path_LA02 path_LA03 path_LA04 path_LA05 path_LA10  ]  = pathdefs();

[ loglikepri_LA02, loglike_LA02, logpri_LA02, max_loglikepri_LA02, Theta_LA02, mu_LA02, sigma_LA02, prec_LA02 ] = loglikepri(path_LA02, '_ROT');
[ loglikepri_LA03, loglike_LA03, logpri_LA03, max_loglikepri_LA03, Theta_LA03, mu_LA03, sigma_LA03, prec_LA03 ] = loglikepri(path_LA03, '_ROT');
[ loglikepri_LA04, loglike_LA04, logpri_LA04, max_loglikepri_LA04, Theta_LA04, mu_LA04, sigma_LA04, prec_LA04 ] = loglikepri(path_LA04, '_ROT');
[ loglikepri_LA05, loglike_LA05, logpri_LA05, max_loglikepri_LA05, Theta_LA05, mu_LA05, sigma_LA05, prec_LA05 ] = loglikepri(path_LA05, '_ROT');
[ loglikepri_LA10, loglike_LA10, logpri_LA10, max_loglikepri_LA10, Theta_LA10, mu_LA10, sigma_LA10, prec_LA10 ] = loglikepri(path_LA10, '_ROT');

max_loglikepri =  max( max( max( max( max_loglikepri_LA02,max_loglikepri_LA03 ),max_loglikepri_LA04 ), max_loglikepri_LA05 ) , max_loglikepri_LA10 ) 

% we divide all the draw by the maximun draw
likepri_LA02 = exp(loglikepri_LA02 - max_loglikepri);
likepri_LA03 = exp(loglikepri_LA03 - max_loglikepri);
likepri_LA04 = exp(loglikepri_LA04 - max_loglikepri);
likepri_LA05 = exp(loglikepri_LA05 - max_loglikepri);
likepri_LA10 = exp(loglikepri_LA10 - max_loglikepri);

logmarginal_LA02  = logmarg(Theta_LA02, likepri_LA02, sigma_LA02, mu_LA02, prec_LA02) +  max_loglikepri;
logmarginal_LA03  = logmarg(Theta_LA03, likepri_LA03, sigma_LA03, mu_LA03, prec_LA03) +  max_loglikepri;
logmarginal_LA10  = logmarg(Theta_LA10, likepri_LA10, sigma_LA10, mu_LA10, prec_LA10) +  max_loglikepri;
logmarginal_LA05  = logmarg(Theta_LA05, likepri_LA05, sigma_LA05, mu_LA05, prec_LA05) +  max_loglikepri;
logmarginal_LA04  = logmarg(Theta_LA04, likepri_LA04, sigma_LA04, mu_LA04, prec_LA04) +  max_loglikepri;

p = [0.1:0.1:0.9];  
a = char('&','&','&','&','&','&','&','&','&');
b = char('\\','\\','\\','\\','\\','\\','\\','\\','\\');
matrix = [ num2str(p') a num2str(logmarginal_LA02','% 10.1f') ...
                       a num2str((logmarginal_LA03-logmarginal_LA02)','% 10.1f')...
                       a num2str((logmarginal_LA04 -logmarginal_LA02)','% 10.1f')...                       
                       a num2str((logmarginal_LA05-logmarginal_LA02)','% 10.1f')...
                       a num2str((logmarginal_LA10-logmarginal_LA02)','% 10.1f') b]
                   
matrix = [ num2str(p') a num2str(logmarginal_LA02','% 10.1f')...
                       a num2str(logmarginal_LA03','% 10.1f')...
                       a num2str(logmarginal_LA04','% 10.1f')...
                       a num2str(logmarginal_LA05','% 10.1f')...
                       a num2str(logmarginal_LA10','% 10.1f') b]